<?php
/**
 * @package     Tag Groups
 *
 * @author      Christoph Amthor
 * @copyright   2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
 * @license     GPL-3.0+
 */

if ( ! class_exists( 'TagGroups_Base' ) ) {

    /**
     *
     */
    class TagGroups_Base {

        /**
         * Returns the first element of an array without changing the original array
         *
         * @param  array   $array
         * @return mixed
         */
        public static function get_first_element( $array = array() ) {

            if ( ! is_array( $array ) ) {

                return;

            }

            return reset( $array );

        }

        /**
         * sanitizes many classes separated by space
         *
         * @param  string   $classes
         * @return string
         */
        public static function sanitize_html_classes( $classes ) {

            // replace multiple spaces by one
            $classes = preg_replace( '!\s+!', ' ', $classes );

            // turn into array
            $classes = explode( ' ', $classes );

            if ( ! empty( $classes ) ) {

                $classes = array_map( 'sanitize_html_class', $classes );

            }

            // turn back
            $classes = implode( ' ', $classes );

            return $classes;

        }

        /**
         * Change the time until the first trial encouragement appears
         *
         * @since 1.19.1
         *
         * @param  int   $sec Default is 24 hours.
         * @return int
         */
        public static function change_time_show_first_trial( $sec ) {

            return 3 * DAY_IN_SECONDS;

        }

        /**
         * Change the time between trial encouragements
         *
         * 
         * @since 1.19.1
         *
         * @param  int   $sec Default is 30 days.
         * @return int
         */
        public static function change_time_reshow_trial( $sec ) {

            // These messages appear only on pages where users go to set up tag groups, not for daily work, that means only a few times per year -> can reshow after 30 days, it effectively will be less

            return 30 * DAY_IN_SECONDS;

        }

        /**
         * Show Freemius admin notice of trial promotion only in Tag Groups own settings or Tag Groups Admin page
         * ("page" parameter starts with tag-groups)
         *
         * @since 1.19.2
         *
         * @param  mixed     $show
         * @param  array     $msg
         * @return boolean
         */
        public static function change_show_admin_notice( $show, $msg ) {

            if (
                'trial_promotion' == $msg['id']
                && ( empty( $_GET['page'] ) || strpos( $_GET['page'], 'tag-groups' ) !== 0 )
            ) {

                // Don't show the trial promotional admin notice.
                return false;

            }

            return true;

        }

        /**
         * Clear the cache of various plugins
         *
         * @param  void
         * @return void
         */
        public static function clear_cache() {

            if ( function_exists( 'flush_pgcache' ) ) {

                flush_pgcache;

            }

            if ( function_exists( 'flush_minify' ) ) {

                flush_minify;

            }

            if ( function_exists( 'wp_cache_clear_cache' ) ) {

                wp_cache_clear_cache();

            }

        }

        /**
         * executes the routines to migrate posts and tags from the base to the premium plugin
         *
         *
         * @param  void
         * @return void
         */
        public static function run_term_migration() {

            global $tag_group_terms;

            /**
             * Add group affiliation to the term meta; term_group will be only secondary
             */

            TagGroups_Error::log( '[Tag Groups] Migrating terms.' );

            $start_time = microtime( true );

            $count_term = $tag_group_terms->convert_to_term_meta();

            TagGroups_Error::log( '[Tag Groups] %d term(s) migrated in %d milliseconds.', $count_term, round( ( microtime( true ) - $start_time ) * 1000 ) );

            if ( $count_term > 0 ) {

                $tag_group_terms->clear_term_cache();

            }

        }

        /**
         * Turns a string into a valid JS function name, preserving as much as possible uniqueness
         *
         * @since 1.26.1
         *
         * @param  string   $raw
         * @return string
         */
        static function create_js_fn_name( $raw ) {

            return str_replace( '-', '', sanitize_html_class( $raw ) );

        }

        /**
         * Check if Gutenberg is active.
         * Must be used not earlier than plugins_loaded action fired.
         * from https://gist.github.com/mihdan/8ba1a70d8598460421177c7d31202908
         *
         * @return bool
         */
        public static function is_gutenberg_active() {
            $gutenberg    = false;
            $block_editor = false;

            if ( has_filter( 'replace_editor', 'gutenberg_init' ) ) {
                // Gutenberg is installed and activated.
                $gutenberg = true;
            }

            if ( version_compare( $GLOBALS['wp_version'], '5.0-beta', '>' ) ) {
                // Block editor.
                $block_editor = true;
            }

            if ( ! $gutenberg && ! $block_editor ) {
                return false;
            }

            include_once ABSPATH . 'wp-admin/includes/plugin.php';

            if ( ! is_plugin_active( 'classic-editor/classic-editor.php' ) ) {
                return true;
            }

            $use_block_editor = ( get_option( 'classic-editor-replace' ) === 'no-replace' );

            return $use_block_editor;

        }

        /**
         * Wrapper for remove_all_filters(), returning previously registered functions
         *
         * @param  array|string $hook_names (array of comma-separated list)
         * @return array
         */
        static function remove_all_filters( $hook_names ) {

            if ( empty( $hook_names ) ) {

                return array();

            }

            if ( ! is_array( $hook_names ) ) {

                $hook_names = array_map( 'trim', explode( ',', $hook_names ) );

            }

            $hook_array = array();

            foreach ( $hook_names as $hook_name ) {

                $hook_array[] = array(
                    'name'        => $hook_name,
                    'subscribers' => self::get_all_hooks( $hook_name ),
                );

                remove_all_filters( $hook_name );

            }

            return $hook_array;

        }

        /**
         * Restores all hook from self::remove_all_filters()
         *
         * @param  array  $hook_array
         * @return void
         */
        static function restore_hooks( $hook_array ) {

            if ( empty( $hook_array ) ) {

                return;
            
            }

            foreach ( $hook_array as $hooks ) {

                foreach ( $hooks['subscribers'] as $subscriber ) {

                    add_filter( $hooks['name'], $subscriber['function'], $subscriber['priority'], $subscriber['accepted_args'] );

                }

            }

        }


        /**
         * List all hooks
         *
         * https://stackoverflow.com/a/26680808
         *
         * @param  string  $hook
         * @return array
         */
        static function get_all_hooks( $hook = '' ) {
            global $wp_filter;

            $hooks = array();

            if ( isset( $wp_filter[$hook]->callbacks ) ) {
                array_walk( $wp_filter[$hook]->callbacks, function ( $callbacks, $priority ) use ( &$hooks ) {

                    foreach ( $callbacks as $id => $callback ) {
                        $hooks[] = array_merge( ['id' => $id, 'priority' => $priority], $callback );
                    }

                } );
            } else {
                return [];
            }

            foreach ( $hooks as &$item ) {

// skip if callback does not exist
                if ( ! is_callable( $item['function'] ) ) {
                    continue;
                }

// function name as string or static class method eg. 'Foo::Bar'
                if ( is_string( $item['function'] ) ) {
                    $ref          = strpos( $item['function'], '::' ) ? new ReflectionClass( strstr( $item['function'], '::', true ) ) : new ReflectionFunction( $item['function'] );
                    $item['file'] = $ref->getFileName();
                    $item['line'] = get_class( $ref ) == 'ReflectionFunction'
                    ? $ref->getStartLine()
                    : $ref->getMethod( substr( $item['function'], strpos( $item['function'], '::' ) + 2 ) )->getStartLine();

                    // array( object, method ), array( string object, method ), array( string object, string 'parent::method' )
                } elseif ( is_array( $item['function'] ) ) {

                    $ref = new ReflectionClass( $item['function'][0] );

                    // $item['function'][0] is a reference to existing object
                    $item['function'] = array(
                        // is_object($item['function'][0]) ? get_class($item['function'][0]) : $item['function'][0], // We need the object! Using the class here leads to "Using $this when not in object context" errors
                        $item['function'][0],
                        $item['function'][1],
                    );
                    $item['file'] = $ref->getFileName();
                    $item['line'] = strpos( $item['function'][1], '::' )
                    ? $ref->getParentClass()->getMethod( substr( $item['function'][1], strpos( $item['function'][1], '::' ) + 2 ) )->getStartLine()
                    : $ref->getMethod( $item['function'][1] )->getStartLine();

                    // closures
                } elseif ( is_callable( $item['function'] ) ) {
                    $ref              = new ReflectionFunction( $item['function'] );
                    $item['function'] = get_class( $item['function'] );
                    $item['file']     = $ref->getFileName();
                    $item['line']     = $ref->getStartLine();
                }

            }

            return $hooks;
        }


        /**
         * Since many front-end features depend on transients, we delete them here if needed
         *
         * @return void
         */
        public static function clear_transients_for_frontend_features() {

            TagGroups_Transients::delete_all_transients( 'tag_groups_group_terms' ); // including variants for languages (appended strings)
  
        }

    }

    // class

}
